#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#include <DHT.h>

#define PRPIN A0
#define MAINB 5
#define PLUSB 4
#define MINUSB 3
#define DHTPIN 12
#define BUZZERPIN 11
#define MOISTPIN A1

#define WET 240
#define DRY 600

#define moistTh 50
#define lumTh 200

enum TempChangeState {Normal, LoValue, HiValue};
TempChangeState tcState = Normal;
float loTempValue = 17;
float hiTempValue = 25;

int luminosity = 400;
float temperature = 25;
int moisture = 60;

int mainBState = 0;
int lastMainBState = 0;

int plusBState = 0;
int lastPlusBState = 0;

int minusBState = 0;
int lastMinusBState = 0;

bool isOnAlarm = false;
int actTone = 300;
int toneSign = 1;

LiquidCrystal_I2C lcd(0x27, 16, 2);
DHT dht(DHTPIN, DHT11);

void lcdPrintMeasurements() {
  if (tcState != Normal)
    return;

  char firstLine[17];
  char secondLine[17];
  char tempStr[8];
  char lumStr[6];
  char moistStr[6];
  char spaceStr[10] = "";

  char loTempStr[8], hiTempStr[8];

  sprintf(lumStr, "%d", luminosity);
  sprintf(moistStr, "%d", moisture);

  dtostrf(temperature, 3, 1, tempStr);
  dtostrf(loTempValue, 3, 1, loTempStr);
  dtostrf(hiTempValue, 3, 1, hiTempStr);
  
  sprintf(firstLine, "T: %s", tempStr);

  int ident = 9 - strlen(lumStr) - strlen(moistStr);
  for(int i = 0; i < ident; i++)
    strcat(spaceStr, " ");
  
  sprintf(secondLine, "L: %s%sM: %s", lumStr, spaceStr, moistStr);
  
  lcd.clear();
  lcd.print(firstLine);
  lcd.print((char)223);
  lcd.print('C');
  
  lcd.setCursor(0, 1);
  lcd.print(secondLine);
  lcd.print((char) 37);
}

void lcdPrintChangeTemp() {
  if (tcState == Normal)
    return;

  if (tcState == LoValue) {
    lcd.clear();
    lcd.print("Valoare inf:");

    char tempStr[8];
    dtostrf(loTempValue, 3, 1, tempStr);
    lcd.setCursor(0, 1);
    lcd.print(tempStr);
  }

  if (tcState == HiValue) {
    lcd.clear();
    lcd.print("Valoare sup:");

    char tempStr[8];
    dtostrf(hiTempValue, 3, 1, tempStr);
    lcd.setCursor(0, 1);
    lcd.print(tempStr);
  }

  lcd.print((char)223);
  lcd.print('C');
}

void lcdPrintError(int type) {
  if (tcState != Normal)
    return;
  
  lcd.clear();
  switch(type) {
    case 1: {
      lcd.print("Temperatura");
      lcd.setCursor(0, 1);
      lcd.print("scazuta!");
      break;
    }
    case 2: {
      lcd.print("Temperatura");
      lcd.setCursor(0, 1);
      lcd.print("ridicata!");
      break;
    }
    case 3: {
      lcd.print("Luminozitate");
      lcd.setCursor(0, 1);
      lcd.print("scazuta!");
      break;
    }
    case 4: {
      lcd.print("Umiditate");
      lcd.setCursor(0, 1);
      lcd.print("scazuta!");
      break;
    }
  }
}

void MainButtonClicked() {
  if (tcState == Normal) {
    tcState = LoValue;
    lcdPrintChangeTemp();
  } else if (tcState == LoValue) {
    tcState = HiValue;
    lcdPrintChangeTemp();
  } else if (tcState == HiValue) {
    tcState = Normal;
    lcdPrintMeasurements();
  }
}

void PlusButtonClicked() {
  if (tcState == Normal)
    return;

  if (tcState == LoValue) {
    loTempValue += 0.5;
  } else if (tcState == HiValue) {
    hiTempValue += 0.5;
  }

  lcdPrintChangeTemp();
}

void MinusButtonClicked() {
  if (tcState == Normal)
    return;

  if (tcState == LoValue) {
    loTempValue -= 0.5;
  } else if (tcState == HiValue) {
    hiTempValue -= 0.5;
  }

  lcdPrintChangeTemp();
}

int checkForAlarm()
{
  if (temperature < loTempValue)
    return 1;
  if (temperature > hiTempValue)
    return 2;

  if (moisture < moistTh)
    return 4;
  if (luminosity < lumTh)
    return 3;

  return 0;
}

void setup() {
  pinMode(PRPIN, INPUT);
  pinMode(MAINB, INPUT_PULLUP);
  pinMode(PLUSB, INPUT_PULLUP);
  pinMode(MINUSB, INPUT_PULLUP);
  pinMode(BUZZERPIN, OUTPUT);
  
  lcd.begin();
  lcd.noBacklight();

  dht.begin();

  Serial.begin(9600);
}

void loop() {
  static unsigned long timerButtons = 0;
  unsigned long intervalButtons = 20;
  
  if (millis() - timerButtons >= intervalButtons)
  {
    timerButtons = millis();
    
    mainBState = digitalRead(MAINB);
    plusBState = digitalRead(PLUSB);
    minusBState = digitalRead(MINUSB);
  
    if (mainBState != lastMainBState)
    {
       if (mainBState == LOW)
       {
          MainButtonClicked();
       }
       lastMainBState = mainBState;
    }

    if (plusBState != lastPlusBState)
    {
       if (plusBState == LOW)
       {
          PlusButtonClicked();
       }
       lastPlusBState = plusBState;
    }

    if (minusBState != lastMinusBState)
    {
       if (minusBState == LOW)
       {
          MinusButtonClicked();
       }
       lastMinusBState = minusBState;
    }

    if (isOnAlarm) {
      tone(BUZZERPIN, actTone);

      if (actTone > 900)
        toneSign = -1;

      if (actTone < 300)
        toneSign = 1; 
      actTone += toneSign * 25;
    } else
      noTone(BUZZERPIN);
  }

  static unsigned long timerSensors = 0;
  unsigned long intervalSensors = 2000;

  if (millis() - timerSensors >= intervalSensors)
  {
    timerSensors = millis();

    luminosity = analogRead(PRPIN);
    float t = dht.readTemperature();

    int moistValue = analogRead(MOISTPIN);
    moisture = map(moistValue, WET, DRY, 100, 0);

    if (!isnan(t))
      temperature = t;

    int ans = checkForAlarm();
    if (ans != 0) {
      isOnAlarm = true;
      lcdPrintError(ans);
    } else {
      isOnAlarm = false;
      lcdPrintMeasurements();
    }
    
  }
}
